#include <io.h>
#include <system.h>
#include <stdio.h>

 
 
 

typedef unsigned int uint;
#define N 2

uint pack(uint red, uint green, uint blue) {
    return ((red&0x3FF)<<20)
        | ((green&0x3FF)<<10)
        | ((blue&0x3FF));
}

void put(int n, int minR, int minG, int minB, int maxR, int maxG, int maxB) {
    IOWR_32DIRECT(COMMUNICATION_0_BASE, 4*n*2,     pack(minR, minG, minB));
    IOWR_32DIRECT(COMMUNICATION_0_BASE, 4*(n*2+1), pack(maxR, maxG, maxB));
}

uint get(int n) {
    return IORD_32DIRECT(COMMUNICATION_0_BASE, 4*n);
}

struct range {
    uint min,max;
};

struct range db[15][4][2]; // { spades, clubs, hearts, diamonds } * 14 cards * {redRange, blackRange}
int vv[15][4];

int cardIndex(char c) {
    switch(c) {
        case 's': case 'S': return 0;
        case 'c': case 'C': return 1;
        case 'h': case 'H': return 2;
        case 'd': case 'D': return 3;
        default: return -1;
    }        
}
const char cardLookup[4] = { 'S', 'C', 'H', 'D' };

uint min(uint a, uint b) {
    if(a<b)
        return a;
    else
        return b;
}

uint max(uint a, uint b) {
    if(a>b)
        return a;
    else
        return b;
}

uint sqClamp(uint a) {
    if(a>=1000)
        return 1000*1000;
    else
        return a*a;
}

void multiGet(struct range col[8], int n) {
    int val,i,j;

    for(i=0;i<8;i++) {
        col[i].min=0xffffffff;
        col[i].max=0;
    }
    
    //printf("Reading");
    for(i=0; i<n;i++) {
        usleep(5000);
        
        for(j=0;j<8;j++) {
            val=get(j);
            col[j].min=min(val,col[j].min);
            col[j].max=max(val,col[j].max);
        }
    }
    //printf(" Done!\n");
}

void waitStable() {
    struct range col[8];
    int i;
    while(1) {
        multiGet(col,500);
        for(i=0;i<8;i++)
            if(col[i].max-col[i].min > 1000)
                break;
        return;
    }
}

int main()
{
    int n,s,i,j;
    uint red,black;
    char c;
    struct range col[8];
         
    memset(vv,0,sizeof(int)*15*4);
    
    put(0, 700,0,0, 1023,500,500);
    put(1, 0,0,0,  500,500,500);  
    
    
    /*while(1) { 
        printf("%u %u\n", get(0), get(1));
        usleep(50000);
    }*/
    
    while(1) {
        printf("Please enter card number (1-14) followed by [s]pade, [c]lub, [h]eart, [d]iamond, and press enter to 'teach' that card: "); 
        scanf("%i%c",&n,&c);
        s=cardIndex(c);

        if(n==0)
            break;

        printf("Reading... ");
        multiGet(col,2000);
        printf("Done!\n");
        db[n][s][0]=col[0];
        db[n][s][1]=col[1];
        vv[n][s]=1;

        db[n][s][0].min*=0.9; db[n][s][0].max*=1.1;
        db[n][s][1].min*=0.9; db[n][s][1].max*=1.1;
                
        printf("red=(%u,%u) black=(%u,%u)\n", db[n][s][0].min,db[n][s][0].max,db[n][s][1].min,db[n][s][1].max);
    }

    
    while(1) {
        //printf("Press enter to recognize card:\n");
        //getchar();
        //getchar();
        waitStable();
        //printf(".");
        
        const int samples = 500;
        uint i,red=0,black=0;
        for(i=0; i<samples; i++) {
            usleep(5000);
            red+=get(0);
            black+=get(1);
        }
        red/=samples;
        black/=samples;
        //printf("Red=%u Black=%u\n",red,black);
        
        uint closest=0xffffffff;
        uint closeN=0, closeS=0;
        for(i=1;i<=14;i++) {
            for(j=0;j<4;j++) {
                if(!vv[i][j])
                    continue;
                if(db[i][j][0].min <= red && red <= db[i][j][0].max &&
                      db[i][j][1].min <= black && black <= db[i][j][1].max) {
                        uint redAvg = (db[i][j][0].min+db[i][j][0].max)/2;
                        uint blackAvg=(db[i][j][1].min+db[i][j][1].max)/2;
                        
                        if( sqClamp(red-redAvg)+sqClamp(black-blackAvg) < closest) {
                            closest=sqClamp(red-redAvg)+sqClamp(black-blackAvg);
                            closeN=i; closeS=j;
                        }
                }
            }
        }
        if(closeN)
                printf("%u %c\n", closeN, cardLookup[closeS]);
    }
    
    return 0;
}


