library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity lab5 is
  
  port (
    signal CLOCK_50 : in std_logic;                -- 50 MHz clock

    SRAM_DQ : inout std_logic_vector(15 downto 0); -- Data bus 16 Bits
    SRAM_ADDR : out std_logic_vector(17 downto 0); -- Address bus 18 Bits
    SRAM_UB_N,                                     -- High-byte Data Mask
    SRAM_LB_N,                                     -- Low-byte Data Mask
    SRAM_WE_N,                                     -- Write Enable
    SRAM_CE_N,                                     -- Chip Enable
    SRAM_OE_N : out std_logic;                      -- Output Enable

    KEY : in std_logic_vector(3 downto 0);         -- Push buttons
    SW : in std_logic_vector(17 downto 0);         -- DPDT switches

    HEX0, HEX1, HEX2, HEX3, HEX4, HEX5, HEX6, HEX7 -- 7-segment displays
       : out std_logic_vector(6 downto 0);
    LEDG : out std_logic_vector(8 downto 0);       -- Green LEDs
    LEDR : out std_logic_vector(17 downto 0);      -- Red LEDs
	GPIO_1 : inout std_logic_vector(35 downto 0);

    -- SDRAM
    DRAM_DQ : inout std_logic_vector(15 downto 0); -- Data Bus
    DRAM_ADDR : out std_logic_vector(11 downto 0); -- Address Bus    
    DRAM_LDQM,                                     -- Low-byte Data Mask 
    DRAM_UDQM,                                     -- High-byte Data Mask
    DRAM_WE_N,                                     -- Write Enable
    DRAM_CAS_N,                                    -- Column Address Strobe
    DRAM_RAS_N,                                    -- Row Address Strobe
    DRAM_CS_N,                                     -- Chip Select
    DRAM_BA_0,                                     -- Bank Address 0
    DRAM_BA_1,                                     -- Bank Address 0
    DRAM_CLK,                                      -- Clock
    DRAM_CKE : out std_logic;                      -- Clock Enable

    -- VGA output   
    VGA_CLK,                                            -- Clock
    VGA_HS,                                             -- H_SYNC
    VGA_VS,                                             -- V_SYNC
    VGA_BLANK,                                          -- BLANK
    VGA_SYNC : out std_logic;                           -- SYNC
    VGA_R,                                              -- Red[9:0]
    VGA_G,                                              -- Green[9:0]
    VGA_B : out std_logic_vector(9 downto 0)            -- Blue[9:0]

  );
    
end lab5;

architecture rtl of lab5 is

    component nios_system is    port (
              -- 1) global signals:
                 signal clk : IN STD_LOGIC;
                 signal reset_n : IN STD_LOGIC;

              -- the_communication_0
                 signal Count0_to_the_communication_0 : IN STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal Count1_to_the_communication_0 : IN STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal Count2_to_the_communication_0 : IN STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal Count3_to_the_communication_0 : IN STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal Count4_to_the_communication_0 : IN STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal Count5_to_the_communication_0 : IN STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal Count6_to_the_communication_0 : IN STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal Count7_to_the_communication_0 : IN STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal max0_from_the_communication_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal max1_from_the_communication_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal max2_from_the_communication_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal max3_from_the_communication_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal max4_from_the_communication_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal max5_from_the_communication_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal max6_from_the_communication_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal max7_from_the_communication_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal min0_from_the_communication_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal min1_from_the_communication_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal min2_from_the_communication_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal min3_from_the_communication_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal min4_from_the_communication_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal min5_from_the_communication_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal min6_from_the_communication_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);
                 signal min7_from_the_communication_0 : OUT STD_LOGIC_VECTOR (31 DOWNTO 0);

              -- the_sram
                 signal SRAM_ADDR_from_the_sram : OUT STD_LOGIC_VECTOR (17 DOWNTO 0);
                 signal SRAM_CE_N_from_the_sram : OUT STD_LOGIC;
                 signal SRAM_DQ_to_and_from_the_sram : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
                 signal SRAM_LB_N_from_the_sram : OUT STD_LOGIC;
                 signal SRAM_OE_N_from_the_sram : OUT STD_LOGIC;
                 signal SRAM_UB_N_from_the_sram : OUT STD_LOGIC;
                 signal SRAM_WE_N_from_the_sram : OUT STD_LOGIC
              );
	end component;

component histo is port (
	clk			: in  std_logic; --50MHz
	R			: in  std_logic_vector(9 downto 0);
	G			: in  std_logic_vector(9 downto 0);
	B			: in  std_logic_vector(9 downto 0);
	new_pixel	: in  std_logic;
	new_frame	: in  std_logic;
	Min			: in  std_logic_vector(31 downto 0);
	Max			: in  std_logic_vector(31 downto 0);
	count		: out std_logic_vector(31 downto 0) );
end component;

component VGA_Controller is port (
	signal iRed : in std_logic_vector(9 downto 0);
	signal iGreen : in std_logic_vector(9 downto 0);
	signal iBlue : in std_logic_vector(9 downto 0);
	signal oRequest : out std_logic;
	--	signal VGA Side
	signal oVGA_R : out std_logic_vector(9 downto 0);
	signal oVGA_G : out std_logic_vector(9 downto 0);
	signal oVGA_B : out std_logic_vector(9 downto 0);
	signal oVGA_H_SYNC : out std_logic;
	signal oVGA_V_SYNC : out std_logic;
	signal oVGA_SYNC : out std_logic;
	signal oVGA_BLANK : out std_logic;
	signal oVGA_CLOCK : out std_logic;
	--	Control Signal
	signal iCLK : in std_logic;
	signal iRST_N : in std_logic );
end component;

component Reset_Delay is port (
	signal iCLK  : in  std_logic;
	signal iRST  : in  std_logic;
	signal oRST_0: out std_logic;
	signal oRST_1: out std_logic;
	signal oRST_2: out std_logic );
end component;

component CCD_Capture is port (
	signal oDATA		: out std_logic_vector( 9 downto 0);
	signal oDVAL		: out std_logic;
	signal oX_Cont		: out std_logic_vector(10 downto 0);
	signal oY_Cont		: out std_logic_vector(10 downto 0);
	signal oFrame_Cont	: out std_logic_vector(31 downto 0);
	signal oNewFrame	: out std_logic;
	signal iDATA		: in  std_logic_vector( 9 downto 0);
	signal iFVAL		: in  std_logic;
	signal iLVAL		: in  std_logic;
	signal iSTART		: in  std_logic;
	signal iEND			: in  std_logic;
	signal iCLK			: in  std_logic;
	signal iRST			: in  std_logic  );
end component;

component RAW2RGB is port (
	signal oRed		: out std_logic_vector( 9 downto 0);
	signal oGreen	: out std_logic_vector( 9 downto 0);
	signal oBlue	: out std_logic_vector( 9 downto 0);
	signal oDVAL	: out std_logic;
	signal iX_Cont	: in  std_logic_vector(10 downto 0);
	signal iY_Cont	: in  std_logic_vector(10 downto 0);
	signal iDATA	: in  std_logic_vector( 9 downto 0);
	signal iDVAL	: in  std_logic;
	signal iCLK		: in  std_logic;
	signal iRST		: in  std_logic		);
end component;

component Sdram_Control_4Port is port (
	--	HOST Side
	signal REF_CLK			: in  std_logic;
	signal RESET_N			: in  std_logic;
	--	FIFO Write Side 1
	signal WR1_DATA			: in  std_logic_vector(15 downto 0);
	signal WR1				: in  std_logic;
	signal WR1_ADDR			: in  std_logic_vector(23 downto 0);
	signal WR1_MAX_ADDR		: in  std_logic_vector(23 downto 0);
	signal WR1_LENGTH		: in  std_logic_vector( 8 downto 0);
	signal WR1_LOAD			: in  std_logic;
	signal WR1_CLK			: in  std_logic;
	signal WR1_FULL			: out std_logic;
	signal WR1_USE			: out std_logic_vector( 8 downto 0);
	--	FIFO Write Side 2
	signal WR2_DATA			: in  std_logic_vector(15 downto 0);
	signal WR2				: in  std_logic;
	signal WR2_ADDR			: in  std_logic_vector(23 downto 0);
	signal WR2_MAX_ADDR		: in  std_logic_vector(23 downto 0);
	signal WR2_LENGTH		: in  std_logic_vector( 8 downto 0);
	signal WR2_LOAD			: in  std_logic;
	signal WR2_CLK			: in  std_logic;
	signal WR2_FULL			: out std_logic;
	signal WR2_USE			: out std_logic_vector( 8 downto 0);
	--	FIFO Read Side 1
	signal RD1_DATA			: out std_logic_vector(15 downto 0);
	signal RD1				: in  std_logic;
	signal RD1_ADDR			: in  std_logic_vector(23 downto 0);
	signal RD1_MAX_ADDR		: in  std_logic_vector(23 downto 0);
	signal RD1_LENGTH		: in  std_logic_vector( 8 downto 0);
	signal RD1_LOAD			: in  std_logic;	
	signal RD1_CLK			: in  std_logic;
	signal RD1_EMPTY		: out std_logic;
	signal RD1_USE			: out std_logic_vector( 8 downto 0);
	--	FIFO Read Side 2
	signal RD2_DATA			: out std_logic_vector(15 downto 0);
	signal RD2				: in  std_logic;
	signal RD2_ADDR			: in  std_logic_vector(23 downto 0);
	signal RD2_MAX_ADDR		: in  std_logic_vector(23 downto 0);
	signal RD2_LENGTH		: in  std_logic_vector( 8 downto 0);
	signal RD2_LOAD			: in  std_logic;	
	signal RD2_CLK			: in  std_logic;
	signal RD2_EMPTY		: out std_logic;
	signal RD2_USE			: out std_logic_vector( 8 downto 0);
	--	SDRAM Side
	signal SA				: out std_logic_vector(11 downto 0);
	signal BA				: out std_logic_vector( 1 downto 0);
	signal CS_N				: out std_logic;
	signal CKE				: out std_logic;
	signal RAS_N			: out std_logic;
	signal CAS_N			: out std_logic;
	signal WE_N				: out std_logic;
	signal DQ				: inout std_logic_vector(15 downto 0);
	signal DQM				: out std_logic_vector( 1 downto 0);
	signal SDR_CLK			: out std_logic		);
end component;


component SEG7_LUT_8 is port (
	signal oSEG0,oSEG1,oSEG2,oSEG3,oSEG4,oSEG5,oSEG6,oSEG7
				: out std_logic_vector( 6 downto 0);
	signal iDIG : in  std_logic_vector(31 downto 0)		);
end component;

component I2C_CCD_Config is	port (
		signal iCLK : IN std_logic;
		signal iRST_N : IN std_logic;
		signal iExposure : IN std_logic_vector(15 downto 0);
		signal I2C_SCLK : OUT std_logic;
		signal I2C_SDAT	: INOUT std_logic );
end component;

component Mirror_Col is port (	--	Input Side
					iCCD_R		: in  std_logic_vector(9 downto 0);
					iCCD_G		: in  std_logic_vector(9 downto 0);
					iCCD_B		: in  std_logic_vector(9 downto 0);
					iCCD_DVAL	: in  std_logic;
					iCCD_PIXCLK	: in  std_logic;
					iRST_N		: in  std_logic;
					--	Output Side
					oCCD_R		: out std_logic_vector(9 downto 0);
					oCCD_G		: out std_logic_vector(9 downto 0);
					oCCD_B		: out std_logic_vector(9 downto 0);
					oCCD_DVAL	: out std_logic		);
end component;


--	CCD
signal	CCD_DATA : std_logic_vector( 9 downto 0);
signal	CCD_SDAT : std_logic;
signal	CCD_SCLK : std_logic;
signal	CCD_FLASH : std_logic;
signal	CCD_FVAL : std_logic;
signal	CCD_LVAL : std_logic;
signal	CCD_PIXCLK : std_logic;
signal	CCD_MCLK : std_logic;	--	CCD Master Clock
signal	Read_DATA1 : std_logic_vector(15 downto 0);
signal	Read_DATA2 : std_logic_vector(15 downto 0);
signal	VGA_CTRL_CLK : std_logic;
signal	AUD_CTRL_CLK : std_logic;
signal	mCCD_DATA : std_logic_vector( 9 downto 0);
signal	mCCD_DVAL : std_logic;
signal	mCCD_DVAL_d : std_logic;
signal	X_Cont : std_logic_vector(10 downto 0);
signal	Y_Cont : std_logic_vector(10 downto 0);
signal	X_ADDR : std_logic_vector(9 downto 0);
signal	Frame_Cont : std_logic_vector(31 downto 0);
signal	mCCD_R : std_logic_vector(9 downto 0);
signal	mCCD_G : std_logic_vector(9 downto 0);
signal	mCCD_B : std_logic_vector(9 downto 0);
signal	DLY_RST_0 : std_logic;
signal	DLY_RST_1 : std_logic;
signal	DLY_RST_2 : std_logic;
signal	Read : std_logic;
signal	rCCD_DATA : std_logic_vector(9 downto 0);
signal	rCCD_LVAL : std_logic;
signal	rCCD_FVAL : std_logic;
signal	sCCD_R : std_logic_vector(9 downto 0);
signal	sCCD_G : std_logic_vector(9 downto 0);
signal	sCCD_B : std_logic_vector(9 downto 0);
signal	sCCD_DVAL : std_logic;
signal  mNewFrame : std_logic;
signal clk : std_logic;

	signal counter : std_logic_vector(15 downto 0);
	signal reset_n : std_logic;
	signal Count0,Count1,Count2,Count3,Count4,Count5,Count6,Count7 : std_logic_vector(31 downto 0);  
	signal min0,min1,min2,min3,min4,min5,min6,min7 : std_logic_vector(31 downto 0);
	signal max0,max1,max2,max3,max4,max5,max6,max7 : std_logic_vector(31 downto 0);

begin

clk <= CLOCK_50;
--  LEDR(17) <= '1';
--  LEDR(16) <= '1';

  process (CLOCK_50)
  begin
    if CLOCK_50'event and CLOCK_50 = '1' then
      if counter = x"ffff" then
        reset_n <= '1';
      else
        reset_n <= '0';
        counter <= counter + 1;
      end if;
    end if;
  end process;

  nios : nios_system port map (
    clk                          => CLOCK_50,
    reset_n                      => reset_n,
	Count0_to_the_communication_0 => Count0,
	Count1_to_the_communication_0 => Count1,
	Count2_to_the_communication_0 => Count2,
	Count3_to_the_communication_0 => Count3,
	Count4_to_the_communication_0 => Count4,
	Count5_to_the_communication_0 => Count5,
	Count6_to_the_communication_0 => Count6,
	Count7_to_the_communication_0 => Count7,
	min0_from_the_communication_0 => min0,
	max0_from_the_communication_0 => max0,
	min1_from_the_communication_0 => min1,
	max1_from_the_communication_0 => max1,
	min2_from_the_communication_0 => min2,
	max2_from_the_communication_0 => max2,
	min3_from_the_communication_0 => min3,
	max3_from_the_communication_0 => max3,
	min4_from_the_communication_0 => min4,
	max4_from_the_communication_0 => max4,
	min5_from_the_communication_0 => min5,
	max5_from_the_communication_0 => max5,
	min6_from_the_communication_0 => min6,
	max6_from_the_communication_0 => max6,
	min7_from_the_communication_0 => min7,
	max7_from_the_communication_0 => max7,
	
	--    leds_from_the_leds           => LEDR(15 downto 0),
    SRAM_ADDR_from_the_sram      => SRAM_ADDR,
    SRAM_CE_N_from_the_sram      => SRAM_CE_N,
    SRAM_DQ_to_and_from_the_sram => SRAM_DQ,
    SRAM_LB_N_from_the_sram      => SRAM_LB_N,
    SRAM_OE_N_from_the_sram      => SRAM_OE_N,
    SRAM_UB_N_from_the_sram      => SRAM_UB_N,
    SRAM_WE_N_from_the_sram      => SRAM_WE_N
  );


	--histo
	h0: histo port map(clk,mCCD_R,mCCD_G,mCCD_B,mCCD_DVAL_d,mNewFrame,min0,max0,Count0);
	h1: histo port map(clk,mCCD_R,mCCD_G,mCCD_B,mCCD_DVAL_d,mNewFrame,min1,max1,Count1);
	h2: histo port map(clk,mCCD_R,mCCD_G,mCCD_B,mCCD_DVAL_d,mNewFrame,min2,max2,Count2);
	h3: histo port map(clk,mCCD_R,mCCD_G,mCCD_B,mCCD_DVAL_d,mNewFrame,min3,max3,Count3);
	h4: histo port map(clk,mCCD_R,mCCD_G,mCCD_B,mCCD_DVAL_d,mNewFrame,min4,max4,Count4);
	h5: histo port map(clk,mCCD_R,mCCD_G,mCCD_B,mCCD_DVAL_d,mNewFrame,min5,max5,Count5);
	h6: histo port map(clk,mCCD_R,mCCD_G,mCCD_B,mCCD_DVAL_d,mNewFrame,min6,max6,Count6);
	h7: histo port map(clk,mCCD_R,mCCD_G,mCCD_B,mCCD_DVAL_d,mNewFrame,min7,max7,Count7);

	CCD_DATA(0)	<=	GPIO_1(0);
	CCD_DATA(1)	<=	GPIO_1(1);
	CCD_DATA(2)	<=	GPIO_1(5);
	CCD_DATA(3)	<=	GPIO_1(3);
	CCD_DATA(4)	<=	GPIO_1(2);
	CCD_DATA(5)	<=	GPIO_1(4);
	CCD_DATA(6)	<=	GPIO_1(6);
	CCD_DATA(7)	<=	GPIO_1(7);
	CCD_DATA(8)	<=	GPIO_1(8);
	CCD_DATA(9)	<=	GPIO_1(9);
	GPIO_1(11)	<=	CCD_MCLK;
--	GPIO_1(15)	<=	CCD_SDAT;
--	GPIO_1(14)	<=	CCD_SCLK;
	CCD_FVAL	<=	GPIO_1(13);
	CCD_LVAL	<=	GPIO_1(12);
	CCD_PIXCLK	<=	GPIO_1(10);
	LEDR		<=	SW;
	LEDG		<=	Y_Cont(8 downto 0);
	VGA_CTRL_CLK<=	CCD_MCLK;


	--halve module
	process (CLOCK_50)
	begin
		if CLOCK_50'event and CLOCK_50 = '1' then
	    	CCD_MCLK <= not CCD_MCLK;
		end if;
	end process;
	
	process (CCD_PIXCLK)
	begin
		if CCD_PIXCLK'event and CCD_PIXCLK='1' then
			rCCD_DATA	<=	CCD_DATA;
			rCCD_LVAL	<=	CCD_LVAL;
			rCCD_FVAL	<=	CCD_FVAL;
		end if;
	end process;

	u1 : VGA_Controller	port map(
							iRed => Read_DATA2(9 downto 0),
							iGreen => Read_DATA1(14 downto 10)&Read_DATA2(14 downto 10),
							iBlue => Read_DATA1(9 downto 0),
							oRequest => Read,
							--	VGA Side
							oVGA_R => VGA_R,
							oVGA_G => VGA_G,
							oVGA_B => VGA_B,
							oVGA_H_SYNC => VGA_HS,
							oVGA_V_SYNC => VGA_VS,
							oVGA_SYNC => VGA_SYNC,
							oVGA_BLANK => VGA_BLANK,
							oVGA_CLOCK => VGA_CLK,
							--	Control Signal
							iCLK => VGA_CTRL_CLK,
							iRST_N => DLY_RST_2	);

	u2: Reset_Delay		port map(
							iCLK => CLOCK_50,
							iRST => KEY(0),
							oRST_0 => DLY_RST_0,
							oRST_1 => DLY_RST_1,
							oRST_2 => DLY_RST_2	);
							
	u3: CCD_Capture		port map(
							oDATA => mCCD_DATA,
							oDVAL => mCCD_DVAL,
							oX_Cont => X_Cont,
							oY_Cont => Y_Cont,
							oFrame_Cont => Frame_Cont,
							oNewFrame	=> mNewFrame,
							iDATA => rCCD_DATA,
							iFVAL => rCCD_FVAL,
							iLVAL => rCCD_LVAL,
							iSTART => '1',
							iEND => not(KEY(2)),
							iCLK => CCD_PIXCLK,
							iRST => DLY_RST_1	);

	u4: RAW2RGB			port map(	
							oRed => mCCD_R,
							oGreen => mCCD_G,
							oBlue => mCCD_B,
							oDVAL => mCCD_DVAL_d,
							iX_Cont => X_Cont,
							iY_Cont => Y_Cont,
							iDATA => mCCD_DATA,
							iDVAL => mCCD_DVAL,
							iCLK => CCD_PIXCLK,
							iRST => DLY_RST_1	);

	u5: SEG7_LUT_8		port map(	
							oSEG0 => HEX0,oSEG1 => HEX1,
							oSEG2 => HEX2,oSEG3 => HEX3,
							oSEG4 => HEX4,oSEG5 => HEX5,
							oSEG6 => HEX6,oSEG7 => HEX7,
							iDIG => Frame_Cont );

	u6: Sdram_Control_4Port port map(	
							--	HOST Side
						    REF_CLK => CLOCK_50,
						    RESET_N => '1',
							--	FIFO Write Side 1
						    WR1_DATA => 	'0'&sCCD_G(9 downto 5)&sCCD_B(9 downto 0),
							WR1 => sCCD_DVAL,
							WR1_ADDR => "000000000000000000000000",
							WR1_MAX_ADDR => "000001010000000000000000",
							WR1_LENGTH => "100000000",
							WR1_LOAD => not(DLY_RST_0),
							WR1_CLK => CCD_PIXCLK,
							--	FIFO Write Side 2
						    WR2_DATA => '0'&sCCD_G(4 downto 0)&sCCD_R(9 downto 0),
							WR2 => sCCD_DVAL,
							WR2_ADDR => "000100000000000000000000",
							WR2_MAX_ADDR => "000101010000000000000000",
							WR2_LENGTH => "100000000",
							WR2_LOAD => not(DLY_RST_0),
							WR2_CLK => CCD_PIXCLK,
							--	FIFO Read Side 1
						    RD1_DATA => Read_DATA1,
				        	RD1 => Read,
				        	RD1_ADDR => "000000000010100000000000",
							RD1_MAX_ADDR => "000000000000000111110000",
							RD1_LENGTH => "100000000",
				        	RD1_LOAD => not(DLY_RST_0),
							RD1_CLK => VGA_CTRL_CLK,
							--	FIFO Read Side 2
						    RD2_DATA => Read_DATA2,
				        	RD2 => Read,
				        	RD2_ADDR => "000100000010100000000000",
							RD2_MAX_ADDR => "000101001101100000000000",
							RD2_LENGTH => "100000000",
				        	RD2_LOAD => not(DLY_RST_0),
							RD2_CLK => VGA_CTRL_CLK,
							--	SDRAM Side
						    SA => DRAM_ADDR,
						    BA(1)=>DRAM_BA_1,
							BA(0)=>DRAM_BA_0,
						    CS_N => DRAM_CS_N,
						    CKE => DRAM_CKE,
						    RAS_N => DRAM_RAS_N,
				            CAS_N => DRAM_CAS_N,
				            WE_N => DRAM_WE_N,
						    DQ => DRAM_DQ,
				            DQM(1) => DRAM_UDQM,
							DQM(0) => DRAM_LDQM,
							SDR_CLK => DRAM_CLK	);

	u7: I2C_CCD_Config 		port map (
							iCLK => CLOCK_50,
							iRST_N => KEY(1),
							iExposure => SW(15 downto 0),
							I2C_SCLK => GPIO_1(14),
							I2C_SDAT => GPIO_1(15) );

	u8: Mirror_Col			port map(	--	Input Side
							iCCD_R => mCCD_R,
							iCCD_G => mCCD_G,
							iCCD_B => mCCD_B,
							iCCD_DVAL => mCCD_DVAL_d,
							iCCD_PIXCLK => CCD_PIXCLK,
							iRST_N => DLY_RST_1,
							--	Output Side
							oCCD_R => sCCD_R,
							oCCD_G => sCCD_G,
							oCCD_B => sCCD_B,
							oCCD_DVAL => sCCD_DVAL);
end rtl;
