library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity led_flasher is
  
  port (
    avs_s1_clk        : in  std_logic;
    avs_s1_reset_n    : in  std_logic;
    avs_s1_read       : in  std_logic;
    avs_s1_write      : in  std_logic;
    avs_s1_chipselect : in  std_logic;
    avs_s1_address    : in  std_logic_vector(4 downto 0);
    avs_s1_readdata   : out std_logic_vector(15 downto 0);
    avs_s1_writedata  : in  std_logic_vector(15 downto 0);

    leds : out std_logic_vector(15 downto 0)
  );
    
end led_flasher;

architecture rtl of led_flasher is

signal clk, reset_n : std_logic;

type ram_type is array(15 downto 0) of std_logic_vector(15 downto 0);

signal RAM : ram_type;
signal ram_address, display_address : std_logic_vector(3 downto 0);
signal counter_delay : std_logic_vector(15 downto 0);
signal counter : std_logic_vector(31 downto 0);

begin

  clk <= avs_s1_clk;
  reset_n <= avs_s1_reset_n;
  ram_address <= avs_s1_address(3 downto 0);

  process (clk)
  begin
    if clk'event and clk = '1' then
      if reset_n = '0' then
        avs_s1_readdata <= (others => '0');
        display_address <= (others => '0');        
        counter <= (others => '0');        
        counter_delay <= (others => '1');
      else
        if avs_s1_chipselect = '1' then
          if avs_s1_address(4) = '0' then
            if avs_s1_read = '1' then
              avs_s1_readdata <= RAM(conv_integer(ram_address));
            elsif avs_s1_write = '1' then
              RAM(conv_integer(ram_address)) <= avs_s1_writedata;
            end if;
          else
            if avs_s1_write = '1' then
              counter_delay <= avs_s1_writedata;
            end if;
          end if;
        else
          leds <= RAM(conv_integer(5));
          if counter = x"00000000" then
            counter <= counter_delay & x"0000";
            display_address <= display_address + 1;
          else
            counter <= counter - 1;
          end if;
        end if;
      end if;
    end if;
  end process;

end rtl;
